import unittest

from ghtrack.Util import Util


class UtilTest(unittest.TestCase):

    def test_is_not_blank(self):
        self.assertFalse(Util.is_not_blank(""), msg="it should be true")
        self.assertTrue(Util.is_not_blank("tests"), msg="tests is not blank")

    def testOneWeekOld(self):
        post = "2014-06-06T22:56:04Z"
        self.assertFalse(Util.oneWeekOld(post))



import json
import logging
from urllib import parse
import requests

from ghtrack.GhTrackException import UnknownApiQueryException
from ghtrack.Util import Util


class RequestInit:
    """This class initialize the requests object with default and required values"""

    def __init__(self, token, apiUrl="https://api.github.com/repos/"):
        """Constructor.
            token of github for unlimited queries if not provided you can not query more than 60 times
            apiUrl the base api url
            :param token: Personal github token .
            :param apiUrl: The base api url.
        """
        self.__tokenHeader = {
            "Accept": "application/vnd.github.v3+json"
        }
        if token:
            self.__tokenHeader["Authorization"] = f"token {token}"
        self.__apiUrl = apiUrl

    """
    This method return the api data in json format for all data not older than the provided param
    :param url: str the :owner/:repo_name
    :param parameters: str not recommended for now
    :param body: str not recommended for now
    :param old: int determines how old the data should be
    :rtype: :tuple:
    """
    def dataRequest(self, url, parameters=None, body="", old: int = 7):
        if parameters is None:
            parameters = dict()

        headers, output = self.__statusCheckedRequest(url, parameters, body)
        # output = [row for row in output if Util.oneWeekOld(row["created_at"], old)]
        # output = list(filter(lambda row: Util.oneWeekOld(row["created_at"], old), dict(output)))

        # page = 2
        # while "link" in headers and "next" in headers["link"]:
        #     parameters["page"] = page
        #     headers, newOutput = self.__statusCheckedRequest(url, parameters, body)
        #     output += newOutput
        #     page += 1

        return output

    """
    This method check the status of request, you can determine if the repo exists.
    :param url: str the :owner/:repo_name
    :param parameters: str not recommended for now
    :param input: str not recommended for now
    :rtype: :int:
    """
    def __statusCheckedRequest(self, url, parameters, input):
        status, headers, output = self.__jsonRequest(url, parameters, input)
        if status < 200 or status >= 300:
            raise UnknownApiQueryException(status, output, headers)
        return headers, output

    """
    This method check the status of request, you can determine if the repo exists.
    :param url: str the :owner/:repo_name
    :param parameters: str not recommended for now
    :param input: str not recommended for now
    :rtype: :int:
    """
    def statusRequest(self, url, parameters, input):
        status, headers, output = self.__jsonRequest(url, parameters, input)
        return status

    """
    This method return the api data in json format
    :param url: str the :owner/:repo_name
    :param parameters: str not recommended for now
    :param input: str not recommended for now
    :rtype: :tuple:
    """
    def __jsonRequest(self, url, parameters, input) -> tuple:
        fullUrl = self.getCompleteUrl(url, parameters)
        try:
            response = requests.get(
                url=fullUrl,
                headers=self.__tokenHeader
            )
            status = response.status_code
            headers = dict(response.headers)
            output = response.json()
            return status, headers, output
        except Exception as ex:
            logging.info(f"Exception during json conversion {ex}")
            return 200, dict(), []



    """
    This method give you the absolute url of the public repo you are querying on
    :param url: str the :owner/:repo_name
    :param parameters: str not recommended for now
    :rtype: :str:
    """
    def getCompleteUrl(self, url, parameters=None):
        if parameters is None or len(parameters) == 0:
            return f"{self.__apiUrl}{url}"
        else:
            return url + "?" + parse.urlencode(parameters)

    """
    In case if you have passed an github token, this method will return the value of the token
    :rtype: :class:`str`
    """
    def getToken(self):
        return self.__tokenHeader





import json


class GhTrackException(Exception):
    """
    This class contains all the possible exceptions that you can encounter during the execution of this project.
    """

    def __init__(self, status, data, headers):
        super().__init__()
        self.__status = status
        self.__data = data
        self.__headers = headers
        self.args = (status, data, headers)

    @property
    def status(self):
        """
        The status returned by the Github API
        """
        return self.__status

    @property
    def data(self):
        """
        The data returned by the Github API
        """
        return self.__data

    @property
    def headers(self):
        """
        The headers returned by the Github API
        """
        return self.__headers

    def __str__(self):
        return "{status} {data}".format(status=self.status, data=json.dumps(self.data))


class NotFoundException(GhTrackException):
    """
    NotFoundException is raised when Github API return 404  status
    """


class BadUserException(GhTrackException):
    """
    BadUserException is raised when request the user which repos you try to pull does not exist.
    """


class RateLimitExceededException(GhTrackException):
    """
    RateLimitExceededException is  raised when the rate limit is exceeded github replies with 403
    """


class RepoNotFoundException(GhTrackException):
    """
    RepoNotFoundException is raised when Github can not find the public repo
    """


class UnknownApiQueryException(GhTrackException):
    """
    UnknownApiQueryException is raised when Github can not find the public repo
    """



import unittest

from ghtrack.RequestInit import RequestInit


class RequestInitTest(unittest.TestCase):

    def setUp(self):
        self.req = RequestInit(token="token")

    def expect(self, url, parameters, input, status, responseHeaders, output):
        pass





from typing import List

from ghtrack.EmailHandler import EmailHandler
from ghtrack.GhTrackObject import GhTrackObject, EmailConf, Repo, Auth

from ghtrack.RequestInit import RequestInit
from ghtrack.Util import Util


class GhTrack(GhTrackObject):
    """Class to pull down the pull request of a github public repositories for a user."""

    def __init__(self, file_name=None, token=None, email=None, user=None, repo=None):
        """Constructor.
            Email uses to send the alert notification.
            User is the repo owner(it can be a user or organisation)
            Repo is the name of the public repo we want to pull down the pull requests.
            :param email: Person to send notification .
            :param user: Github user or organization.
            :param repo: Github public repo.
            :param file_name is the absolute path of your configuration file
        """
        super().__init__(filename=file_name)
        self.alertEmail = self.emailConf if email is None else EmailConf(to=email)
        self.repoInfos = self.repoConf if user is None or repo is None else Repo(user=user, repo=repo)
        self.authToken = self.authToken if token is None else Auth(token=token)
        self.public_repo = self.__user_repo(self.repoInfos)
        self.ghRequest = RequestInit(token=self.authToken.token)
        self.age = 7
        # self.emailNotConsole = False
        self.emailHandler = EmailHandler()
        self.pullRequests = self.getPulls()

    """
        This method build the full name of the public repository :owner/:reponame
        :param number: :class:`ghtrack.GhTrackObject.Repo`
        :rtype: `str`
    """

    def __user_repo(self, repo: Repo):
        return f"{repo.user}/{repo.repo}"

    """
    This method return the public repo from the config file and that you passed during the object creation
    :rtype: :class:`str`
    """

    def getRepo(self):
        return self.ghRequest.dataRequest(url=self.public_repo)

    """
    This method retrieves an all pull requests younger than the age provided, by default it 7 days
    :rtype: :class:`List[dict]`
    """

    def getPulls(self):
        pulls = self.ghRequest.dataRequest(url=f"{self.public_repo}/pulls", old=self.age)
        return list(filter(lambda row: Util.oneWeekOld(row["created_at"], self.age), pulls))

    """
    This method retrieves an individual pull request by it number
    :param number: int
    :rtype: :class:`dict`
    """

    def getPull(self, number):
        # / repos /: owner /:repo / pulls /: number
        assert number is not None
        return self.ghRequest.dataRequest(url=f"{self.public_repo}/pulls/{number}")

    """
    This method sets the age of the pull requests to retrieve, by default it is 7 days
    :param age: int
    :rtype: :class:`None`
    """

    def setAge(self, age: int):
        self.age = age

    """
    This method builds the summary to be sent by email or to print on the conssole
    :param pullRequests: List[dict]
    :rtype: str
    """

    def __getSummary(self, pullRequests: List[dict]) -> str:
        output = ""
        for v in pullRequests:
            content = f""""
                <h1><a href="{v["html_url"]}">{v['title']} - state: {v['state']}<a></h1>
                <p><strong>Open date:<strong>{v["created_at"]}<p> 
            """
            output += content
        return output

    """
    This method send the summary to be sent by email
    :param pullRequests: List[dict]
    :rtype: str
    """

    def __sendEmail(self, content: str) -> bool:
        code, body, header = self.emailHandler.sendEmail(content)
        if 200 <= code <= 299:
            return True
        return False

    """
    This method print the summary or send the summary by email
    :param emailNotConsole: bool
    :rtype: str
    """

    def sendEmailOrPrintConsole(self, emailNotConsole: bool = False) -> str:
        content = self.__getSummary(self.pullRequests)
        if emailNotConsole and self.emailConf.sendGridApi:
            res = self.__sendEmail(content)
            if res:
                return f"Email was successfully sent from {self.emailConf.fromEmail} to {self.emailConf.to}, with subject {self.emailConf.subject}"
        return content

    """
    This method retrieve the json form of pull requests based on the status
    :param status: Set[str] ("open", "closed")
    :rtype: List[dict]
    """

    def getPullsByStatus(self, status: str):
        if not (status.lower() in ("open", "closed")):
            raise Exception("A valid params is either open or closed")
        return [row for row in self.getPulls() if Util.isOpenOrClose(row["state"])]


# if __name__ == '__main__':
#     params = "created=2021-08-30..2021-09-01"
#     g = GhTrack()
#     g.setAge(0)
#     print(g.repoInfos.repo)
#     res = g.getPullsByStatus("open")
#     print(len(res))
#     print(g.getPullsByStatus("open"))
#     # res = g.getPulls()
#     # print(g.emailHandler)
#     # print(g.emailHandler.emailConf.sendGridApi)
#     print(g.sendEmailOrPrintConsole(True))



import logging

from ghtrack.GhTrackObject import EmailConf, GhTrackObject
from sendgrid import SendGridAPIClient
from sendgrid.helpers.mail import Mail


class EmailHandler(GhTrackObject):
    """EmailHandler is use to send email through twilio."""

    def __repr__(self):
        return f"to={self.emailConf.to}, from={self.emailConf.fromEmail}, subject={self.emailConf.subject}"

    """
    This method send email to an configurable email
    :param content: str
    :rtype: :tuple
    """
    def sendEmail(self, content: str) -> tuple:
        # try:
        message = Mail(
            from_email=self.emailConf.fromEmail,
            to_emails=self.emailConf.to,
            subject=self.emailConf.subject,
            html_content=content)

        sg = SendGridAPIClient(api_key=self.emailConf.sendGridApi)
        response = sg.send(message)
        return response.status_code, response.body, response.headers
        # except Exception as ex:
        #     logging.info(f"Api key = {self.sendGridApi}")
        #     logging.info(f"exception = {ex}")




import logging
from datetime import datetime


class Util:

    @staticmethod
    def is_not_blank(s) -> bool:
        return bool(s and not s.isspace())

    @staticmethod
    def oneWeekOld(postDate: str, old: int = 7) -> bool:
        post = datetime.fromisoformat(postDate[:-1])
        diffDay = (datetime.now() - post).days
        return diffDay <= old

    @staticmethod
    def isOpenOrClose(state: str) -> bool:
        return state.lower() in ("open", "closed")




#!/usr/bin/env python3
import argparse
from colorama import Fore, Back, Style

from ghtrack.GhTrack import GhTrack

"""
github-track.commandline
~~~~~~~~~~~~~~~~~~~~~~
github-track is a module that pulls the 7 days latest github pull requests.
This module is it command line interface
Usage:
    >>> ghtrack --help
    usage: ghtrack [-h] token
    positional arguments:
      token    Github Token
    optional arguments:
      -h, --help  show this help message and exit
    >>> ghtrack <token>
    # set a token for unlimited queries.
    >>> ghtrack
    # if you dont see any token, but with a rate limit of 60.
    >>> ghtrack <reponame>
    #  pulls the 7 days latest github pull requests.
"""


class CommandLineUtil:

    @staticmethod
    def main():
        """Starting point for the program execution."""

        # Create command line parser.
        parser = argparse.ArgumentParser()

        # Adding command line arguments.
        parser.add_argument("-f", dest="file", help="Config file", default=None, type=str, required=False)

        # Adding command line arguments.
        parser.add_argument("-t", dest="token", help="Github token", default=None, type=str, required=False)

        # Adding command line arguments.
        parser.add_argument("-e", dest="email", help="The email to send alert", default="zinatestmail@gmail.com", type=str, required=False)

        # Adding command line arguments.
        parser.add_argument("-u", dest="user", help="The public repo owner", default="kubernetes", type=str, required=False)

        # Adding command line arguments.
        parser.add_argument("-r", dest="repo", help="The public repo", default="kubernetes", type=str, required=False)

        # Parse command line arguments.
        arguments = parser.parse_args()
        print(arguments)
        file, token, email, user, repo = arguments.file, arguments.token, arguments.email, arguments.user, arguments.repo
        print(Fore.CYAN, "With token you will have an higher rate queries")
        print(Fore.CYAN, "Skipping token may result with rate limit exception")
        print(Fore.GREEN, f"file = {file}, token = {token}, email = {email}, user = {user}, repo = {repo}")
        print(Style.RESET_ALL)
        try:
            print(Style.BRIGHT)
            g = GhTrack(file_name=file, token=token, email=email, user=user, repo=repo)
            # pullRequests = g.getPulls()
            emailNotConsole = False
            if emailNotConsole:
                print(Fore.GREEN, "Since you did not provide an email, here the console")
                print(Fore.RED, f"Public repo: {g.public_repo}")
                print(Fore.RED, "~~~~~~~~~~SUMMARY~~~~~~~~~")
                print(Fore.MAGENTA, f"{g.sendEmailOrPrintConsole(emailNotConsole)}")
                print(Style.RESET_ALL)
            else:
                print(Fore.GREEN, f"Since you provided an email, an email was sent to: {g.alertEmail.to}")
                print(Fore.RED, f"Public repo: {g.public_repo}")
                print(Fore.RED, "~~~~~~~~~~SUMMARY SENT BY EMAIL~~~~~~~~~")
                print(Fore.MAGENTA, f"{g.sendEmailOrPrintConsole(emailNotConsole)}")
                print(Style.RESET_ALL)
        except Exception as ex:
            print(Style.DIM)
            print(Fore.RED, f"Something went wrong during the object initialization {ex}")
            print(Style.RESET_ALL)
            exit()




import os
from dataclasses import dataclass
import yaml

import logging as log

@dataclass
class Auth:
    """Auth class.
        Github token configurations.
    """
    token: str = ""


@dataclass
class EmailConf:
    """EmailConf class.
        Email configurations.
    """
    to: str = "zinalacina@gmail.com"
    subject: str = "Pull request Test"
    sendGridApi: str = ""
    fromEmail: str = "zlacina@gmail.com"


@dataclass
class Repo:
    """Repo class.
        Public repo setting.
    """
    user: str = "kubernetes"
    repo: str = "kubernetes"


@dataclass
class GhTrackObject:
    """GhTrackObject class.
        Contents the configuration information coming from data/config.yml and also default value.
    """
    filename: str = None
    emailConf: EmailConf = EmailConf()
    repoConf: Repo = Repo()
    authToken: Auth = Auth()

    def __post_init__(self):
        self.emailConf, self.repoConf, self.authToken = self.getConf(self.filename)

    def getConf(self, file_name: str) -> (EmailConf, Repo, Auth):
        if not file_name:
            file_name = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "data/config.yml")
        else:
            file_name = os.path.abspath(file_name)
        try:
            with open(file_name) as f:
                setting = yaml.safe_load(f)["settings"]
                token = Auth(setting["github"]["token"])
                email = EmailConf(to=setting["email"]["to"],
                                  subject=setting["email"]["subject"],
                                  sendGridApi=setting["email"]["sendGridApi"],
                                  fromEmail=setting["email"]["from"])
                repo = Repo(user=setting["repo"]["user"], repo=setting["repo"]["name"])
                return email, repo, token
        except Exception as ex:
            log.info(f"impossible to open file {file_name}, ex={ex}")
            return EmailConf(), Repo(), Auth()





import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="GhTrack",
    version="0.0.1",
    py_modules=["GhTrack"],
    description="An Python module api app",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zinaLacina/github-track",
    project_urls={
        "Bug Tracker": "https://github.com/zinaLacina/github-track/issues",
    },
    package_data={'': ['data/*']},
    test_suite="tests",
    author="Lacina ZINA",
    author_email="zinalacina@yahoo.com",
    package_dir={"": "ghtrack"},
    packages=setuptools.find_packages(where="ghtrack"),

    install_requires=[
        "requests~=2.26.0",
        "PyYAML~=5.4.1",
        "sendgrid==6.8.1",
        "colorama==0.4.4"
    ],
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: The Unlicense (Unlicense)",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
        "Typing :: Typed",
    ],
)
